part of 'one_gang_glass_switch_bloc.dart';

@immutable
abstract class OneGangGlassSwitchState {}

class OneGangGlassSwitchInitial extends OneGangGlassSwitchState {}

class OneGangGlassSwitchLoading extends OneGangGlassSwitchState {}

class OneGangGlassSwitchStatusLoaded extends OneGangGlassSwitchState {
  final OneGangGlassStatusModel status;

  OneGangGlassSwitchStatusLoaded(this.status);
}

class OneGangGlassSwitchError extends OneGangGlassSwitchState {
  final String message;

  OneGangGlassSwitchError(this.message);
}

class OneGangGlassSwitchBatchStatusLoaded extends OneGangGlassSwitchState {
  final OneGangGlassStatusModel status;

  OneGangGlassSwitchBatchStatusLoaded(this.status);
}

class OneGangGlassSwitchBatchControlError extends OneGangGlassSwitchState {
  final String message;

  OneGangGlassSwitchBatchControlError(this.message);
}
